@@ -11,7 +11,7 @@ func findAuthPinFilePath(username string) (string, error) {
 	slog.Info("getting datapath for", "user", username)
 	dataPath, err := getDataPath(fsFriendlyJid(username), true)
 	if err != nil {
-		return strEmpty, fmt.Errorf("find auth pin file path: %v", err)
+		return strEmpty, fmt.Errorf("find auth pin file path: %w", err)
 	}
 	return fmt.Sprintf("%sauthpin", dataPath), nil
 }
@@ -20,13 +20,13 @@ func parseAuthPinFile(username string) (bool, error) {
 	// Find auth pin file.
 	authPinFile, err := findAuthPinFilePath(username)
 	if err != nil {
-		return false, fmt.Errorf("parse auth pin file: %v", err)
+		return false, fmt.Errorf("parse auth pin file: %w", err)
 	}
 	// Read file.
 	slog.Info("reading auth pin", "file", authPinFile)
 	f, err := os.ReadFile(authPinFile)
 	if err != nil {
-		return false, fmt.Errorf("parse auth pin file: %v", err)
+		return false, fmt.Errorf("parse auth pin file: %w", err)
 	}
 	// Strip trailing newline.
 	content := strings.TrimSuffix(string(f), "\n")
@@ -39,7 +39,7 @@ func writeAuthPinFile(username string) error {
 	// Find auth pin file.
 	authPinFile, err := findAuthPinFilePath(username)
 	if err != nil {
-		return fmt.Errorf("write auth pin file: %v", err)
+		return fmt.Errorf("write auth pin file: %w", err)
 	}
 	slog.Info("writing auth pin", "file", authPinFile)
 	err = os.WriteFile(authPinFile, []byte("noPLAIN\n"), 0o400)
