@@ -24,11 +24,16 @@ import (
 	"github.com/xmppo/go-xmpp"                 // BSD-3-Clause
 )
 
-func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems, drc chan xmpp.DiscoResult,
+func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems, drc chan xmpp.DiscoResult, slc chan xmpp.Slot,
 	jserver string, filePaths []string, timeout time.Duration, oxPrivKey *crypto.Key, keyRing *crypto.KeyRing,
 ) (urls []string, err error) {
-	var uploadComponent string
-	var maxFileSize int64
+	var (
+		uploadComponent string
+		maxFileSize     int64
+		req             *http.Request
+	)
+
+	buffer := new(bytes.Buffer)
 
 	// Query server for disco#items
 	slog.Info("http-upload: querying disco items", "server", jserver)
@@ -82,23 +87,37 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems
 	}
 	for _, filePath := range filePaths {
 		// Read file
-		buffer, err := readFile(filePath)
+		file, err := os.Open(filePath)
 		if err != nil {
-			return urls, err
+			return urls, fmt.Errorf("http-upload: read file: %w", err)
 		}
-		fileSize := buffer.Len()
+		defer file.Close()
+
+		fileStat, err := file.Stat()
+		if err != nil {
+			return urls, fmt.Errorf("http-upload: stat file: %w", err)
+		}
+		fileSize := fileStat.Size()
 		slog.Info("http-upload: checking file size:", "file", filePath, "size", fileSize)
 		// Get mime type
-		mimeType := mimetype.Detect(buffer.Bytes()).String()
+		mtype, err := mimetype.DetectFile(filePath)
+		if err != nil {
+			return urls, fmt.Errorf("http-upload: detect mimetype: %w", err)
+		}
+		mimeType := mtype.String()
 		slog.Info("http-upload:", "mime type", mimeType)
 		var mimeTypeEscaped bytes.Buffer
 		xml.Escape(&mimeTypeEscaped, []byte(mimeType))
 		if oxPrivKey != nil {
+			_, err = readFile(filePath)
+			if err != nil {
+				return nil, fmt.Errorf("http-upload: %w", err)
+			}
 			buffer, err = legacyPGPEncryptFile(oxPrivKey, keyRing, buffer)
 			if err != nil {
 				return urls, err
 			}
-			fileSize = buffer.Len()
+			fileSize = int64(buffer.Len())
 			slog.Info("http-upload: updated file size after encrypting:", "file", filePath, "size", fileSize)
 		}
 		// Get file name
@@ -136,35 +155,15 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems
 
 		// Request http upload slot
 		slog.Info("http-upload: requesting upload slot")
-		uploadSlot, err := sendIQ(client, iqc, uploadComponent, "get", r)
+		uploadSlot, err := getSlot(client, slc, uploadComponent, r)
 		if err != nil {
 			return urls, err
 		}
-		if uploadSlot.Type != strResult {
-			return urls, fmt.Errorf("http-upload: error while requesting upload slot")
-		}
-		iqHTTPUploadSlotXML := etree.NewDocument()
-		err = iqHTTPUploadSlotXML.ReadFromBytes(uploadSlot.Query)
-		if err != nil {
-			return urls, err
-		}
-		iqHTTPUploadSlotXMLSlot := iqHTTPUploadSlotXML.SelectElement("slot")
-		if iqHTTPUploadSlotXMLSlot == nil {
-			return urls, fmt.Errorf("http-upload: no slot element")
-		}
-		iqHTTPUploadSlotXMLPut := iqHTTPUploadSlotXMLSlot.SelectElement("put")
-		if iqHTTPUploadSlotXMLPut == nil {
-			return urls, fmt.Errorf("http-upload: no put element")
-		}
-		iqHTTPUploadSlotXMLPutURL := iqHTTPUploadSlotXMLPut.SelectAttr("url")
-		if iqHTTPUploadSlotXMLPutURL == nil {
-			return urls, fmt.Errorf("http-upload: no url attribute")
-		}
-		if !strings.HasPrefix(iqHTTPUploadSlotXMLPutURL.Value, "https://") {
+		if !strings.HasPrefix(uploadSlot.Put.Url, "https://") {
 			return urls, fmt.Errorf("http-upload: upload slot does not provide https")
 		}
 		// Upload file
-		slog.Info("http-upload: uploading file to", "slot", iqHTTPUploadSlotXMLPutURL.Value)
+		slog.Info("http-upload: uploading file to", "slot", uploadSlot.Put.Url)
 		httpTransport := &http.Transport{
 			IdleConnTimeout:     timeout,
 			TLSHandshakeTimeout: timeout,
@@ -180,21 +179,22 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems
 			httpTransport.Proxy = http.ProxyURL(proxyURL)
 		}
 		httpClient := &http.Client{Transport: httpTransport}
-		req, err := http.NewRequest(http.MethodPut, iqHTTPUploadSlotXMLPutURL.Value,
-			buffer)
+		if oxPrivKey != nil {
+			req, err = http.NewRequest(http.MethodPut, uploadSlot.Put.Url, buffer)
+		} else {
+			req, err = http.NewRequest(http.MethodPut, uploadSlot.Put.Url, file)
+		}
 		if err != nil {
 			return urls, err
 		}
 		req.Header.Set("Content-Type", mimeTypeEscaped.String())
-		iqHTTPUploadSlotXMLPutHeaders := iqHTTPUploadSlotXMLPut.SelectElements("header")
-		for _, h := range iqHTTPUploadSlotXMLPutHeaders {
-			name := h.SelectAttr("name")
-			if name == nil {
+		for _, h := range uploadSlot.Put.Headers {
+			if h.Name == strEmpty {
 				continue
 			}
-			switch name.Value {
+			switch h.Name {
 			case "Authorization", "Cookie", "Expires":
-				req.Header.Set(name.Value, h.Text())
+				req.Header.Set(h.Name, h.Value)
 			}
 		}
 		resp, err := httpClient.Do(req)
@@ -208,20 +208,19 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, disc chan xmpp.DiscoItems
 		}
 
 		// Return http link
-		iqHTTPUploadSlotXMLGet := iqHTTPUploadSlotXMLSlot.SelectElement("get")
-		if iqHTTPUploadSlotXMLGet == nil {
-			return urls, fmt.Errorf("http-upload: no get element")
-		}
-		iqHTTPUploadSlotXMLGetURL := iqHTTPUploadSlotXMLGet.SelectAttr("url")
-		if iqHTTPUploadSlotXMLGetURL == nil {
+		if uploadSlot.Get.Url == strEmpty {
 			return urls, fmt.Errorf("http-upload: no url attribute")
 		}
-		slog.Info("http-upload: received get", "URL", iqHTTPUploadSlotXMLGetURL.Value)
+		if !strings.HasPrefix(uploadSlot.Get.Url, "https://") {
+			return urls, fmt.Errorf("http-upload: get url does not provide https")
+		}
+		slog.Info("http-upload: received get", "URL", uploadSlot.Get.Url)
 		err = resp.Body.Close()
 		if err != nil {
 			fmt.Println("http-upload: error while closing http request body:", err)
 		}
-		urls = append(urls, iqHTTPUploadSlotXMLGetURL.Value)
+		urls = append(urls, uploadSlot.Get.Url)
+		file.Close()
 	}
 	return urls, nil
 }
